;-----------------------------------------------------------------------
; AutoHotkey script to create an animated GIF from photobooth photos.
;
; Usage: Double click on this script to run it
; A title screen can be displayed (after the photos) by saving a file named GIF_title.jpg in the
; images folder. Please note the GIF_title.jpg image must have the same aspect ratio as
; the photos otherwise you may see parts of the last photo when the title screen is displayed.
; An optional overlay can be added to each photo by saving a file named GIF_overlay_1.png,
; GIF_overlay_2.png, GIF_overlay_3.png etc. in the photobooth images folder. If the numbered file
; isn't found the script will look for GIF_overlay.png instead.
; The GIF animation files will be saved in a subfolder named GIF of your images folder.
;
; AutoHotKey is free and can be downloaded from:
; http://www.autohotkey.com
;
; This script comes with no warranty or support whatsoever and may
; be freely copied or modified as required.
;
; Written by Chris Breeze, www.breezesys.com
; 20131019: Updated to handle pathnames containing spaces
; 20140517: Modified to monitor a folder for JPEGs
; 20151111: Added reverseDisplay option to loop display e.g. 1, 2, 3, 4, 3, 2, 1, 2, 3, 4...
; 20160225: Added option to also create an MP4 copy of the animated GIF
; 20200901: Updated ffmpegDir to use the Flipbook Photobooth installation folder
;-----------------------------------------------------------------------
#SingleInstance,force
#NoEnv
SetWorkingDir %A_ScriptDir%

; set this to 1 to enable debug messages or 0 to disable them
showDebugMessages := 0

; resize photos to this width (in pixels)
resizeWidth := 600

; how long to display title screen (in 1/100 sec)
titleDelay := 200

; how long to display each photo (in 1/100 sec)
imageDelay := 10

; set reverseDisplay to 1 to add middle photos to the end of the loop in reverse order e.g. 1, 2, 3, 4, 3, 2
reverseDisplay := 0

; create MP4 copy of animated GIF
createMP4copy := 1
transcodeUsingHandbrakeCLI := 0
ffmpegDir = C:\Program Files (x86)\BreezeSys\Flip Book Photobooth
if createMP4copy
{
	ifnotexist,%ffmpegDir%\ffmpeg.exe
	{
		MsgBox, ffmpeg not found, please check ffmpeg is installed and ffmpegDir is defined in the script.`nffmpegDir=%ffmpegDir%
		exitapp
	}
	if transcodeUsingHandbrakeCLI
	{
		ifnotexist,%A_ScriptDir%\HandBrakeCLI.exe
		{
			MsgBox, HandBrakeCLI.exe not found, please download a copy from https://handbrake.fr/downloads2.php and save it in %A_ScriptDir%
			exitapp
		}
	}
}

if showDebugMessages
{
	Gui +LastFound +AlwaysOnTop +ToolWindow  ; +ToolWindow avoids a taskbar button and an alt-tab menu item.
	Gui, Color, cBlack
	Gui, Font, s12 
	Gui, Add, Text, vStatusText cLime, AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	Gui, Add, Text, vStatusText2 cLime, AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	Gui, Show,x100 y0 NoActivate,%A_ScriptName%  ; NoActivate avoids deactivating the currently active window.
	GuiControl,,StatusText,Please wait...
	GuiControl,,StatusText2,
}

Loop
{
	ControlGetText, txt, Static1, Flip Book Photobooth
	imageDir := RegExReplace(txt, ".*Images saved to:\s*(.*[^\s])\s*Screen images in:.*$", "$1")
	gifDir := imageDir . "\GIF"

	IfNotExist, %imageDir%
	{
		DebugMessage(1, "Not found: " . imageDir)
		DebugMessage(2, "Wait for directory...")
		Sleep 5000
		continue
	}
	DebugMessage(1, "Image folder: " . imageDir . "...")
	FileCreateDir, %gifDir%
	IfNotExist, %gifDir%
	{
		DebugMessage(2, "Error creating GIF dir: " . gifDir)
		Sleep 5000
		continue
	}
	DebugMessage(2, "Looking for new files to convert...")

	; scan image folders
	Loop, %imageDir%\20*, 2
	{
		; look for folders with page_01.jpg but no GIF file
		gifFile := gifDir . "\" . A_LoopFileName . ".GIF"
		movFile := gifDir . "\" . A_LoopFileName . ".MOV"
		page1 := A_LoopFileFullPath . "\Page_01.jpg"
		images =
		IfNotExist,%gifFile%
		{
			IfExist,%page1%
			{
				photoDir := A_LoopFileFullPath
				DebugMessage(2, "Processing: " . photoDir . "...")

				; Look for title page named GIF_title.jpg or title.jpg
				title = %imageDir%\GIF_title.jpg
				IfNotExist,%title%
				{
					title = %imageDir%\title.jpg
				}

				; get a list of files to process
				image_count = 0
				Loop, 1000
				{
					if A_Index < 10
					{
						photo := "frame_0" . A_index . ".tif"
					}
					else
					{
						photo := "frame_" . A_index . ".tif"
					}
					IfNotExist,%photoDir%\%photo%
					{
						break
					}

					; look for overlay for this image (e.g. GIF_overlay_1.png) else GIF_overlay.png
					overlay = %imageDir%\GIF_overlay_%A_Index%.png
					IfNotExist,%overlay%
					{
						overlay = %imageDir%\GIF_overlay_0%A_Index%.png
					}
					IfNotExist,%overlay%
					{
						overlay = %imageDir%\GIF_overlay.png
					}
					IfExist,%overlay%
					{
						images%A_Index% = ( "%photo%" "%overlay%" -resize %resizeWidth% -flatten )
					}
					else
					{
						images%A_Index% = %images% "%photo%"
					}
					image_count++
				}

				; construct the ImageMagick convert command
				cmd = "%A_ScriptDir%\convert.exe"

				; loop GIF animation forever and add the photos
				cmd = %cmd% -loop 0 -delay %imageDelay%
				loop %image_count%
				{
					cmd := cmd . " " . images%A_Index%
				}
				if reverseDisplay
				{
					; add reverse images (except first and last)
					loop %image_count%
					{
						index := image_count - A_Index
						if (index = 1)
						{
							break
						}
						cmd := cmd . " " . images%index%
					}
				}

				; add title screen if defined
				ifExist, %title%
				{
					cmd = %cmd% -delay %titleDelay% "%title%"
				}

				cmd = %cmd% -resize %resizeWidth% "%gifFile%"

				; run the command
				runwait %cmd%, %photoDir%, Hide

				if createMP4copy
				{
					if transcodeUsingHandbrakeCLI
					{
						; create a temporary MOV fie using ffmpeg and transcode using HandBrakeCLI to make it Instagram friendly
						FileDelete temp.mov
						cmd = %ffmpegDir%\ffmpeg.exe -f gif -i "%gifFile%" -vcodec mjpeg temp.mov
						runwait %cmd%, %dir%, Hide
						cmd = %A_ScriptDir%\HandBrakeCLI.exe -i temp.mov -o "%movFile%"
						runwait %cmd%, %dir%, Hide
						FileDelete temp.mov
					}
					else
					{
						cmd = %ffmpegDir%\ffmpeg.exe -f gif -i "%gifFile%" -vcodec mjpeg "%movFile%"
						runwait %cmd%, %dir%, Hide
					}
				}

				Sleep, 1000
			}
		}
	}
	Sleep, 1000
	DebugMessage(2, "")
	Sleep, 5000
}

DebugMessage(num, msg)
{
	global cStatusText, cStatusText2
	global showDebugMessages
	if showDebugMessages
	{
		if num=1
		{
			GuiControl,,StatusText,%msg%
		}
		else
		{
			GuiControl,,StatusText2,%msg%
		}
	}
}
